﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.ConditionActionEncoding {
  [Item("ModuloOperator", "Description missing")]
  [StorableClass]
  public class ModuloOperator : SingleSuccessorOperator {

    #region Parameter Properties
    public ILookupParameter<IntValue> LeftSideParameter {
      get { return (ILookupParameter<IntValue>)Parameters["LeftSide"]; }
    }
    public ILookupParameter<IntValue> RightSideParameter {
      get { return (ILookupParameter<IntValue>)Parameters["RightSide"]; }
    }
    public ILookupParameter<IntValue> ResultParameter {
      get { return (ILookupParameter<IntValue>)Parameters["Result"]; }
    }
    #endregion

    [StorableConstructor]
    protected ModuloOperator(bool deserializing) : base(deserializing) { }
    protected ModuloOperator(ModuloOperator original, Cloner cloner)
      : base(original, cloner) {
    }
    public ModuloOperator()
      : base() {
      Parameters.Add(new LookupParameter<IntValue>("LeftSide", ""));
      Parameters.Add(new LookupParameter<IntValue>("RightSide", ""));
      Parameters.Add(new LookupParameter<IntValue>("Result", ""));
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new ModuloOperator(this, cloner);
    }

    public override IOperation Apply() {
      if (ResultParameter.ActualValue == null) {
        ResultParameter.ActualValue = new IntValue(LeftSideParameter.ActualValue.Value % RightSideParameter.ActualValue.Value);
      } else {
        ResultParameter.ActualValue.Value = LeftSideParameter.ActualValue.Value % RightSideParameter.ActualValue.Value;
      }
      return base.Apply();
    }
  }
}
